package com.ca.actions.startup;

import static org.junit.Assert.fail;

import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.sql.Connection;
import java.util.Collections;
import java.util.Date;
import java.util.Properties;

import javax.sql.DataSource;

import org.apache.log4j.Level;
import org.h2.jdbcx.JdbcDataSource;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.teiid.adminapi.impl.ModelMetaData;
import org.teiid.jdbc.TeiidDriver;
import org.teiid.runtime.EmbeddedConfiguration;
import org.teiid.runtime.EmbeddedServer;
import org.teiid.translator.TranslatorException;
import org.teiid.translator.jdbc.h2.H2ExecutionFactory;

import com.ca.chorus.db.AContinuousQueryResultsListener;
import com.ca.chorus.db.DbExecutor;
import com.ca.chorus.db.QueryControl;
import com.ca.chorus.db.Tuple;
import com.ca.chorus.server.translator.async.TimerangeExecutionFactory;
import com.google.common.eventbus.DeadEvent;
import com.google.common.eventbus.Subscribe;
import com.google.common.util.concurrent.ListenableFuture;

public class AlertsGroupingContinuousQueriesTests
{

    private static EmbeddedServer                                       server;

    private static EmbeddedServer.ConnectionFactoryProvider<DataSource> jdbcProvider;

    static DbExecutor                                                   teiidDbExecutor;

    static DbExecutor                                                   h2DbExecutor;

    private static JdbcDataSource                                       h2ds;

    @BeforeClass
    public static void setUp() throws Exception
    {
        initH2Database();
        final EmbeddedConfiguration ec = new EmbeddedConfiguration();
        ec.setUserRequestSourceConcurrency(1);
        ec.setUseDisk(true);
        server = new EmbeddedServer();
        server.start(ec);
        server.addConnectionFactoryProvider("source-jdbc", jdbcProvider);
        server.addTranslator(new H2ExecutionFactory());
        buildDeployVDB(server);

        final TeiidDriver td = server.getDriver();
        final Connection c = td.connect("jdbc:teiid:Chorus", null);
        teiidDbExecutor = new DbExecutor(c);

    }

    @AfterClass
    public static void tearDown() throws Exception
    {

        h2DbExecutor.commitAndClose();
        teiidDbExecutor.commitAndClose();
        server.stop();
    }

    @SuppressWarnings("unchecked")
    @Ignore
    @Test
    public void testSingeRowInsertion() throws Exception
    {
        // Below query is the one which gets fired to retrieve the severity
        // counts

        // SELECT CHORUS_B.AlertsPriority, COUNT(CHORUS_B.AlertsPriority) AS
        // CHORUS_GROUPING_COUNT, 'GSV.ZOSALERTS' AS "__objecttype__"
        // FROM (SELECT CHORUS_B.* FROM (SELECT * FROM GSV.ZOSALERTS) AS
        // CHORUS_B,
        // (SELECT * FROM chorus_time.timetable WHERE period = 30000) AS
        // chorus_refresh_expression) AS CHORUS_B GROUP BY
        // CHORUS_B.AlertsPriority

        final String sql = "SELECT CHORUS_B.NAME, COUNT(CHORUS_B.NAME) AS CHORUS_GROUPING_COUNT "
                           + "FROM (SELECT CHORUS_B.* FROM (SELECT * FROM TEST) AS CHORUS_B,"
                           + " (CALL chorus_time.timetable(10000)) AS chorus_refresh_expression) AS CHORUS_B GROUP BY CHORUS_B.NAME";

        DbExecutor.QUERY_LOGGER.setLevel(Level.DEBUG);
        teiidDbExecutor.setPlanWriter(new PrintWriter(new FileOutputStream("C:\\QueryPlan.txt")));

        final ListenableFuture<Void> future = teiidDbExecutor.executeContinuously(sql,
                                                                                  Collections.EMPTY_LIST,
                                                                                  new AContinuousQueryResultsListener<Object>()
                                                                                  {

                                                                                      @Override
                                                                                      public void onTuple(final QueryControl<Object> queryControl,
                                                                                                          final Tuple tuple,
                                                                                                          final boolean isFirst,
                                                                                                          final boolean isLastOfResultSetOrBatch) throws Exception
                                                                                      {
                                                                                          System.out.println("Executing :"
                                                                                                             + new Date()
                                                                                                             + " tuple:"
                                                                                                             + tuple);

                                                                                      }
                                                                                  });

        future.get();

    }

    private static void initH2Database() throws Exception
    {
        h2ds = new JdbcDataSource();
        h2ds.setURL("jdbc:h2:target/testInsert");

        jdbcProvider = new EmbeddedServer.ConnectionFactoryProvider<DataSource>()
        {
            @Override
            public DataSource getConnectionFactory() throws TranslatorException
            {
                return AlertsGroupingContinuousQueriesTests.h2ds;
            }
        };
        final Connection conn = jdbcProvider.getConnectionFactory()
                                            .getConnection();

        h2DbExecutor = new DbExecutor(conn);

        h2DbExecutor.execute("DROP SCHEMA IF EXISTS TESTSCHEMA");
        h2DbExecutor.execute("CREATE SCHEMA TESTSCHEMA");
        h2DbExecutor.execute("CREATE TABLE TESTSCHEMA.TEST(ID INT PRIMARY KEY, NAME VARCHAR(255))");
        h2DbExecutor.execute("INSERT INTO TESTSCHEMA.TEST (ID, NAME) VALUES (1, 'abc')");
        h2DbExecutor.execute("INSERT INTO TESTSCHEMA.TEST (ID, NAME) VALUES (2, 'def')");
        h2DbExecutor.execute("INSERT INTO TESTSCHEMA.TEST (ID, NAME) VALUES (3, 'def')");
        h2DbExecutor.execute("INSERT INTO TESTSCHEMA.TEST (ID, NAME) VALUES (4, 'abc')");
        h2DbExecutor.execute("INSERT INTO TESTSCHEMA.TEST (ID, NAME) VALUES (5, 'abc')");
    }

    private static void buildDeployVDB(final EmbeddedServer teiidServer) throws Exception
    {

        final ModelMetaData h2ContinuousModel = new ModelMetaData();

        final Properties props = new Properties();
        props.setProperty("importer.schemaPattern", "TESTSCHEMA");
        h2ContinuousModel.setProperties(props);

        h2ContinuousModel.setName("h2ContinuousTranslator");
        h2ContinuousModel.setSchemaSourceType("native");
        h2ContinuousModel.addSourceMapping("h2ContinuousTranslator",
                                           "h2",
                                           "source-jdbc");

        final TimerangeExecutionFactory timerangeExecutionFactory = new TimerangeExecutionFactory();
        final ModelMetaData timeModel = new ModelMetaData();
        timeModel.setName("chorus_time");
        timeModel.setSchemaSourceType("native");
        timeModel.addSourceMapping("chorus_time",
                                   "timerange-async",
                                   "source-file");
        server.addTranslator(timerangeExecutionFactory);

        teiidServer.deployVDB("Chorus", h2ContinuousModel, timeModel);
    }

    @Subscribe
    public void handleDeadEvent(final DeadEvent deadEvent)
    {
        fail("dead event: " + deadEvent);
    }

}
